package stratego;


public abstract class Motor
{
	protected Tablero board = null;
	protected Estados status = Estados.STOPPED;
	protected int turn = 0;

	public Pieza getBoardPiece(int x, int y)
	{
		return board.getPiece(new Puntero(x, y));
	}
	
	public Pieza getTrayPiece(int i)
	{
		return board.getTrayPiece(i);
	}

	public int getTraySize()
	{
		return board.getTraySize();
	}
	
	public int getTurn()
	{
		return turn;
	}
	
	private boolean move(Movimiento m)
	{
		if (status == Estados.PLAYING)
		{
			if (m.getFrom().equals(Tablero.IN_TRAY))
				return false;
			if (m.getTo().equals(Tablero.IN_TRAY))
				return false;
			
			if (board.attack(m))
				return true;
	
			return board.move(m);
		}
		return false;
	}
	
	public void newGame()
	{
		board.clear();
		turn = 0;
		status = Estados.SETUP;
		
		if (Ajustes.bShowAll)
			board.showAll();
	}
	
	public void play()
	{
		if (status != Estados.SETUP)
			return;
		
		if (board.getTraySize() == 0)
			status = Estados.PLAYING;
		
		if (Ajustes.bShowAll)
			board.showAll();
	}


	protected boolean requestMove(Movimiento m)
	{
		if (m == null || !move(m))
			return false;
		
		if (status == Estados.PLAYING)
		{
			int winner;
			if ((winner = board.checkWin()) >= 0)
			{
				status = Estados.STOPPED;
				board.showAll();
				gameOver(winner);
			}

			turn = (turn + 1) % 2;
		}

		update();
		return true;
	}
	
	public boolean setupPlacePiece(Pieza p, Puntero s)
	{
		if (status == Estados.SETUP)
			return board.add(p, s);
		
		return false;
	}
	
	public boolean setupRemovePiece(Puntero s)
	{
		if (status == Estados.SETUP)
			return board.remove(s);
		
		return false;
	}
	
	protected abstract void gameOver(int winner);
	protected abstract void update();
}
